#include <windows.h>
#include <stdio.h>

using namespace std;

bool isDllRegistered(const char* dllName);
bool registerDll(const char* dll);

int main(int argc, char *argv[])
{
  char dll[255];
  
  printf("Path to the component (DLL or OCX): ");
  while(fgets(dll, 255, stdin) == NULL);
  
  if (registerDll(dll))
  {
    printf("The file '%s' has been successfully registered !\n", dll);
  }
  else
  {
    int length = strlen(dll);
    int index = length-1;
    while(index > -1)
    {
      if (dll[index] == '/' ||
          dll[index] == '\\') break;
      --index;
    }
    
    char* dllName = (char*)malloc(sizeof(char)*(length-index-1));
    strncpy(dllName, dll+index+1, length-index-2);
    dllName[length-index-2] = '\0';
    
    if(isDllRegistered(dllName)) printf("The component '%s' was found in the registery (it is registered)\n", dllName);
    else printf("The component '%s' is not registered to the system.\nDid you run this program as administrator ?\n", dllName);
    
    free(dllName);
  }

  char x;
  printf("Hit enter to exit...");
  scanf("%c",&x);
}

bool registerDll(const char* dll)
{
  HINSTANCE hDLL = LoadLibrary(dll);
  if(NULL == hDLL) return FALSE;
  
  typedef HRESULT (CALLBACK *HCRET)(void);
  HCRET lpfnDllRegisterServer;
  
  lpfnDllRegisterServer = (HCRET)GetProcAddress(hDLL, "DllRegisterServer");
  if(lpfnDllRegisterServer == NULL) return FALSE;
  
  if(FAILED((*lpfnDllRegisterServer)())) return FALSE;
  
  return TRUE;
}

bool isDllRegistered(const char* dllName)
{
  bool found = FALSE;
  HKEY clsidKey;
  
  const char* clsidPath = "SOFTWARE\\Classes\\CLSID";
  const char* clsidInsideKeyPath = "\\InprocServer32";
  const int clsidPathSize = strlen(clsidPath);
  const int clsidInsideKeyPathSize = strlen(clsidInsideKeyPath);
  
  if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, clsidPath, 0, KEY_READ, &clsidKey) == ERROR_SUCCESS)
  {
    int keyNameSize = 255;
    char keyName[255];
    int keyIndex = 0;
    while(RegEnumKey(clsidKey, keyIndex, keyName, keyNameSize) == ERROR_SUCCESS && !found)
    {
      //build the full path => SOFTWARE\Classes\CLSID\{1234-5678-1234-5678}\InprocServer32
      const int keySize = strlen(keyName);
      const int fullPathSize = clsidPathSize+clsidInsideKeyPathSize+keySize+1/* the slash */;
      char* fullPath = (char*)malloc(sizeof(char)*(fullPathSize+1));
      strncpy(fullPath, clsidPath, clsidPathSize);
      strncpy(fullPath+clsidPathSize+1, keyName, keySize);
      strncpy(fullPath+clsidPathSize+1+keySize, clsidInsideKeyPath, clsidInsideKeyPathSize);
      fullPath[clsidPathSize] = '\\';
      fullPath[fullPathSize] = '\0';
      
      //open the path and get the value
      HKEY fullPathKey;
      if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, fullPath, 0, KEY_READ, &fullPathKey) == ERROR_SUCCESS)
      {
        unsigned long size = 255;
        char keyValue[255];
        if(RegQueryValueEx(fullPathKey, 0, 0, 0, (BYTE*)keyValue, &size)  == ERROR_SUCCESS)
        {
          char* vcfCheck = strstr(keyValue, dllName);
          if (vcfCheck != NULL) found = TRUE;
        }
        RegCloseKey(fullPathKey);
      }
                             
      free(fullPath);
      ++keyIndex;
    }
    RegCloseKey(clsidKey);
  }
  
  return found;
}
